"""A module containing all Azol models"""
from .generic_resource import GenericResource
from .devops_rsa_parameters import DevOpsRSAParameters

__all__ = [
    "GenericResource",
    "DevOpsRSAParameters"
]
